f = File.open("jitter_args.c", 'a+')
of = File.open("temp.txt",'w')

f.rewind
if f.eof?
	f.puts "//setter/getter declarations\n"
	f.puts "\n//attribute variables\n"
	f.puts "\n//setters/getters\n"
	f.puts "\n//attribute registration\n"
	f.puts "\n//attribute initialization\n"
	f.rewind
end

type = "char"
jit_type = "_jit_sym_char"
list_length = 1
att_name = "dummy"
has_setter = false
has_getter = false
has_filter = false
setter_name = "0L"
getter_name = "0L"
external_name = "my_jit_external"

ARGV.each do |i|
	if i == '-l'
		type = "long"
		jit_type = "_jit_sym_long"
	elsif i == '-f'
		type = "float"
		jit_type = "_jit_sym_float32"
	elsif i == '-d'
		type = "double"
		jit_type = "_jit_sym_float64"
	elsif i == '-a'
		type = "t_atom"
		jit_type = "_jit_sym_atom"
	elsif i == '-s'
		type = "t_symbol*"
		jit_type = "_jit_sym_symbol"
	elsif i == '-get'
		has_getter = true
	elsif i == '-set'
		has_setter = true
	elsif i == '-clip'
		has_filter = true
	elsif i.to_i > 1
		list_length = i.to_i
	elsif i.match("^-")
		external_name = i[/.(.*)/m,1].gsub('.','_')
	else
		att_name = i;
	end
end

setter_name = "#{external_name}_set_#{att_name}" if has_setter
getter_name = "#{external_name}_get_#{att_name}" if has_getter

f.each do |l|
	of.puts l
	##Attribute variables
	if l.include? "variables"
		of.puts type + " " + att_name + ";" if list_length == 1
		of.puts "long #{att_name}count;\n" + type + " " + att_name + "[#{list_length}];" if list_length > 1
	##Setter/getter declarations
	elsif l.include? "declarations"
		if has_setter
			of.puts "t_jit_err #{setter_name}(t_#{external_name} *x, void *attr, long ac, t_atom *av);"
		end
		if has_getter
			of.puts "t_jit_err #{getter_name}(t_#{external_name} *x, void *attr, long *ac, t_atom **av);"
		end
	##Setters/getters
	elsif l.include? "getters"
		if has_setter
			of.puts "t_jit_err #{setter_name}(t_#{external_name} *x, void *attr, long ac, t_atom *av){\n"
			of.puts "\tif(ac < #{list_length}){"
			of.puts "\t\t//Not enough parameters?"
			of.puts "\t\treturn JIT_ERR_NONE;\n\t}\n"
			of.puts "\n\treturn JIT_ERR_NONE;\n}\n\n"
		end
		if has_getter
			of.puts "t_jit_err #{getter_name}(t_#{external_name} *x, void *attr, long *ac, t_atom **av){\n\n"
			of.puts "\tint i;\n" if list_length > 1
			of.puts "\tif ((*ac)&&(*av)) {"
			of.puts "\t\t//memory passed in, use it"
			of.puts "\t} else {\n\t\t//otherwise allocate memory"
			of.puts "\t\t*ac = #{list_length};"
			of.puts "\t\tif (!(*av = jit_getbytes(sizeof(t_atom)*(*ac)))) {"
			of.puts "\t\t\t*ac = 0;"
			of.puts "\t\t\treturn JIT_ERR_OUT_OF_MEM;\n\t\t}\n\t}\n\n"
		
			if list_length > 1
				of.puts "\tfor(i=0;i<#{list_length};i++)jit_atom_setlong(av[i],(long)x->#{att_name}[i]);\n" if type == "char"
				of.puts "\tfor(i=0;i<#{list_length};i++)jit_atom_setlong(av[i],x->#{att_name}[i]);\n" if type == "long"
				of.puts "\tfor(i=0;i<#{list_length};i++)jit_atom_setfloat(av[i],x->#{att_name}[i]);\n" if type == "float"
				of.puts "\tfor(i=0;i<#{list_length};i++)jit_atom_setfloat(av[i],(float)x->#{att_name}[i]);\n" if type == "double"
				of.puts "\tfor(i=0;i<#{list_length};i++)jit_atom_setsym(av[i],x->#{att_name}[i]);\n" if type == "t_symbol*"
				of.puts "\tfor(i=0;i<#{list_length};i++)av[i] = x->#{att_name}[i];\n" if type == "t_atom"
			else
				of.puts "\tjit_atom_setlong(*av,(long)x->#{att_name});\n" if type == "char"
				of.puts "\tjit_atom_setlong(*av,x->#{att_name});\n" if type == "long"
				of.puts "\tjit_atom_setfloat(*av,x->#{att_name});\n" if type == "float"
				of.puts "\tjit_atom_setfloat(*av,(float)x->#{att_name});\n" if type == "double"
				of.puts "\tjit_atom_setsym(*av,x->#{att_name});\n" if type == "t_symbol*"
				of.puts "\t*av = x->#{att_name};\n" if type == "t_atom"
			end
			
			of.puts "\n\treturn JIT_ERR_NONE;\n}\n\n"
		
		end
	##registration
	elsif l.include? "registration"
		if list_length > 1
			of.puts "attr = (t_jit_object *)jit_object_new(_jit_sym_jit_attr_offset_array, \"#{att_name}\", #{jit_type}, #{list_length}, \n\tattrflags, (method)#{getter_name},(method)#{setter_name}, \n\tcalcoffset(t_#{external_name}, #{att_name}count),calcoffset(t_#{external_name},#{att_name}));"
		else
			of.puts"attr = (t_jit_object *)jit_object_new(_jit_sym_jit_attr_offset,\"#{att_name}\",#{jit_type},\n\tattrflags,(method)#{getter_name},(method)#{setter_name},calcoffset(t_#{external_name},#{att_name}));"
		end
		of.puts "jit_attr_addfilterset_clip(attr,0,1,TRUE,TRUE);" if has_filter
		of.puts "jit_class_addattr(_#{external_name}_class,attr);\n\n"
	##initialization
	elsif l.include? "initialization"
		if list_length > 1
			list_length.times do |i|
				type == "t_atom" ? of.puts("jit_atom_setlong(&x->#{att_name}[#{i}],0);") : type == "t_symbol*" ? of.puts("x->#{att_name}[#{i}] = gensym(\"a_symbol\");") : of.puts("x->#{att_name}[#{i}] = 0;")
			end
		else
			type == "t_atom" ? of.puts("jit_atom_setlong(&x->#{att_name},0);") : type == "t_symbol*" ? of.puts("x->#{att_name} = gensym(\"a_symbol\");") : of.puts("x->#{att_name} = 0;")
		end
	end
end




File.delete("jitter_args.c")
File.rename("temp.txt","jitter_args.c")